// File query_tree.h
///////////////////
//  Query trees represent queries to a text data base.
#ifndef QUERY_TREE_H
#define QUERY_TREE_H

#include "function-ll.h"
#include "set-ll.h"

#include <iostream>
#include "ch_string.h"

using namespace std ;

typedef Set<ch_string>                     StringSet ;
typedef Function< ch_string, StringSet >   Index ;

class QueryNode // An ABSTRACT class!
{
    public :
        // Evaluate the query for a given database.
        virtual StringSet evaluate( const Index &index, const StringSet &all) = 0 ;

        // Print the query
        virtual void print( ostream &out )  = 0 ;
        virtual void printFF( )  = 0 ;
} ;

class StringNode : public QueryNode
{
    public :
        // Constuctor
        StringNode( ch_string str ) { string_ = str ; }

        // Evaluate the query for a given database.
        virtual StringSet evaluate( const Index &index, const StringSet &all) ;

        // Print the query
        virtual void print( ostream &out ) ;
        virtual void printFF( ) ;

    private :
        ch_string string_ ;
} ;

class AndNode : public QueryNode
{
    public :
        // Constructor
        AndNode( QueryNode *leftChild, QueryNode *rightChild ) 
            { leftChild_ = leftChild ; rightChild_ = rightChild ; }

        // Evaluate the query for a given database.
        virtual StringSet evaluate( const Index &index, const StringSet &all) ;

        // Print the query
        virtual void print( ostream &out ) ;
        virtual void printFF( );

    private :
        QueryNode *leftChild_ , *rightChild_ ;
} ;

class OrNode : public QueryNode
{
    public :
        // Constructor
        OrNode( QueryNode *leftChild, QueryNode *rightChild )
            { leftChild_ = leftChild ; rightChild_ = rightChild ; }

        // Evaluate the query for a given database.
        virtual StringSet evaluate( const Index &index, const StringSet &all) ;

        // Print the query
        virtual void print( ostream &out ) ;
        virtual void printFF( ) ;

    private :
        QueryNode *leftChild_ , *rightChild_ ;
} ;

class NotNode : public QueryNode
{
    public :
        // Constructor
        NotNode( QueryNode *child ) 
            { child_ = child ; }

        // Evaluate the query for a given database.
        virtual StringSet evaluate( const Index &index, const StringSet &all) ;

        // Print the query
        virtual void print( ostream &out ) ;
        virtual void printFF( );

    private :
        QueryNode *child_ ;
} ;

#endif