/* File: test_query.cpp
   Author:  T.S. Norvell
   Update:  0.  22/June/1998.  First revision.
   Update:  1.  24/June/1998.  7 tests completed.
   Update:  2.  18/June/2004. 
*/

/* Test jig for the querytree class.
*/

#include "query_tree.h" 

#include <cstdlib>
#include <ctime>
#include <iostream>  // Standard I/O

using namespace std ;

static ch_string ffwords = "www.fwords.com" ;
static ch_string fmusic = "www.music.com" ;
static ch_string funk = "www.unkious.com" ;
 
void makeTestDatabase( Function< ch_string, Set<ch_string> >  &database ) 
{
    // File 0 
    {   Set< ch_string> dataFile ;
        ch_string fileName =  ffwords ;
        dataFile.insert( "foo" ) ;
        dataFile.insert( "firkin" ) ;
        dataFile.insert( "farthing" ) ;
        dataFile.insert( "filtrum" ) ;
        dataFile.insert( "fizzle" ) ;
        dataFile.insert( "fjord" ) ;
        dataFile.insert( "funk" ) ;

        database.add( fileName, dataFile ) ;
    }

    // File 1 
    {   Set< ch_string> dataFile ;
        ch_string fileName = fmusic ;
        dataFile.insert( "funk" ) ;
        dataFile.insert( "rock" ) ;
        dataFile.insert( "jazz" ) ;
        dataFile.insert( "classical" ) ;
        dataFile.insert( "tap" ) ;
        dataFile.insert( "country" ) ;
        dataFile.insert( "punk" ) ;

        database.add( fileName, dataFile ) ;
    }

    // File 2 
    {   Set< ch_string> dataFile ;
        ch_string fileName = funk ;
        dataFile.insert( "funk" ) ;
        dataFile.insert( "flunk" ) ;
        dataFile.insert( "sunk" ) ;
        dataFile.insert( "spunk" ) ;
        dataFile.insert( "lunk" ) ;
        dataFile.insert( "hunk" ) ;
        dataFile.insert( "punk" ) ;

        database.add( fileName, dataFile ) ;
    }
}

void makeIndex( Function< ch_string, StringSet > database,
                Index                         &index,
                StringSet                     &all ) {

    // Process each file in the data base.
    while( ! database.empty() ) {
        ch_string fileName ;
        StringSet contents ;

        database.getAny( fileName, contents ) ;

        // Add the file name to the set of all filenames.
        all.insert( fileName ) ;

        // Add fileName to the index set of each word in contents.
        // The `index set' of a word will be the set of all file(name)s
        // it is in.
        while( ! contents.empty() ) {
            ch_string word = contents.getAny() ;
            if( index.inDomain( word ) ) {
                // Add the file name to the set of filenames for this word.
                StringSet fNameSet = index.at( word ) ;
                fNameSet.insert( fileName ) ;
                index.remove( word ) ;
                index.add( word, fNameSet ) ; }
            else {
                // Add a new word to the index
                StringSet fNameSet ;
                fNameSet.insert( fileName ) ;
                index.add( word, fNameSet ) ; }

            contents.remove( word ) ; }

        database.remove( fileName ) ; }
}

void printStringSet( StringSet S ) {
    cout << "{" ;
    if( ! S.empty() ) {
        while( S.size() > 1 ) {
            ch_string someMember = S.getAny() ;
            cout << someMember << ", " ;
            S.remove( someMember ) ; }
        cout << S.getAny() ; }
    cout << "}" ; }

void printFunct( Function< ch_string, StringSet>  database ) {
    while( ! database.empty() ) {
        ch_string fileName ;
        StringSet contents ;
        database.getAny( fileName, contents ) ;
        cout << fileName << " --> " ;
        printStringSet( contents ) ;
        cout << "\n" ;
        database.remove( fileName ) ; } }

// testCount --- the number of tests //
///////////////////////////////////////
int testCount() { return 7 ; }

// doTest  --- do one test //
/////////////////////////////
void doTest( int testNumber, int &result ) {
    try {
        bool pass = true ;
        cout << "Test " << testNumber << "\n" ;
        bool ok = false ;

        Function< ch_string, StringSet > database ;
        makeTestDatabase( database ) ;

        Index index ;
        StringSet all ;
        makeIndex( database, index, all ) ;

        static bool printed = false ;
        if( ! printed ) {
            cout << "Database is:\n" ;
            printFunct( database ) ;
            cout << "\n" ;
            cout << "Index is:\n" ;
            printFunct( index ) ;
            cout << "\n" ; 
            printed = true ; }
     
        StringNode qfunk( "funk" ) ;
        StringNode qfjord( "fjord" ) ;
        StringNode qblah( "blah" ) ;
        StringNode qfiltrum( "filtrum" ) ;
        StringNode qpunk( "punk" ) ;
        StringNode qfirkin( "firkin" ) ;
        StringNode qtap( "tap" ) ;
        StringNode qjazz( "jazz" ) ;
        
        NotNode    qNOTblah( &qblah ) ;
        
        OrNode     qfiltrumORjazz( & qfiltrum, & qjazz ) ;
        AndNode    qpunkAND_filtrumORjazz( & qpunk, & qfiltrumORjazz ) ;
        
        NotNode    qNOTfirkin( &qfirkin ) ;
        OrNode     qtapORNOTfirkin( &qtap, &qNOTfirkin ) ;

        NotNode    qNOTtap( &qtap ) ;
        AndNode    qNOTtap_AND_firkin( &qNOTtap, &qfirkin ) ;
        NotNode    qNOT_NOTtapANDfirkin( &qNOTtap_AND_firkin ) ;

        switch( testNumber ) {

        case 0 : {     
            StringSet myResult = all ;
            QueryNode *q = & qfunk ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;
 
        case 1 : {     
            StringSet myResult ;
            myResult.insert( ffwords ) ;
            QueryNode *q = & qfjord ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;

        case 2 : {     
            StringSet myResult ; // Empty
            QueryNode *q = & qblah ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;

        case 3 : {     
            StringSet myResult = all ;
            QueryNode *q = & qNOTblah ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;

        case 4 : {     
            StringSet myResult ; myResult.insert( fmusic ) ;
            QueryNode *q = & qpunkAND_filtrumORjazz ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;

        case 5 : {     
            StringSet myResult ;
            myResult.insert( fmusic ) ; myResult.insert( funk ) ;
            QueryNode *q = & qtapORNOTfirkin ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;

        case 6 : {    
            StringSet myResult ;
            myResult.insert( fmusic ) ; myResult.insert( funk ) ;
            QueryNode *q = & qNOT_NOTtapANDfirkin ;
            cout << "NB. This is DeMorgan's law applied to test 5\n" ;
            cout << "Query is: "; q->print( cout ) ; cout << "\n" ;
            cout << "Query is: "; q->printFF( ) ; cout << "\n" ;
            StringSet yourResult = q->evaluate( index, all ) ;
            cout << "Your result = " ; printStringSet( yourResult ) ; cout << "\n" ;
            cout << "My result   = " ; printStringSet( myResult ) ; cout << "\n" ;
            pass = myResult==yourResult ;
        } break ;
       default : pass = false ;
        } 

        if( pass ) {
            cout << "Test " << testNumber << " passes\n" ;
            result = 0 ; }
        else {
            cout << "Test " << testNumber << " fails\n" ;
            result = 1 ; } }
    catch( ... ) {
        cerr << "Sorry an unanticipated exception occurred\n" ;
        result = 1 ; } }